package com.citygis.gis.tool.domain.component.interfaces;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.citygis.common.boot.enums.SystemStatusCodeEnum;
import com.citygis.common.boot.exception.BusinessException;
import com.citygis.common.boot.struct.Result;
import com.citygis.common.boot.util.DirectoryUtil;
import com.citygis.common.boot.util.PathUtil;
import com.citygis.gis.tool.domain.component.define.Property;
import com.citygis.gis.tool.domain.constant.Constant;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

public abstract class BaseAlgorithm implements Algorithm {
    /**
     * 输入参数元数据
     */
    protected Map<String, Property> inputParametersMeta = new LinkedHashMap<>();
    /**
     * 输出参数元数据
     */
    protected Map<String, Property> outputParametersMeta = new LinkedHashMap<>();
    /**
     * 特征参数元数据
     */
    protected Map<String, Property> featureParametersMeta = new LinkedHashMap<>();
    /**
     * 是否完成初始化
     */
    protected Boolean initComplete = false;

    @Override
    public Map<String, Property> getInputParams() {
        return this.inputParametersMeta;
    }

    @Override
    public Map<String, Property> getOutputParams() {
        return this.outputParametersMeta;
    }

    @Override
    public Map<String, Property> getFeatureParams() {
        return this.featureParametersMeta;
    }

    @Override
    public abstract void init(Map<String, Property> parameters);

    @Override
    public Map<String, Object> getMetadata() {
        Map<String, Object> metadata = new HashMap<>();
        metadata.put("InputParameters", inputParametersMeta);
        metadata.put("OutputParameters", outputParametersMeta);
        metadata.put("FeatureParameters", featureParametersMeta);
        return metadata;
    }

    @Override
    public abstract Result execute();

    /**
     * 验证参数集合
     *
     * @param parameters
     * @return
     */
    protected Boolean verifyParameters(Map<String, Property> parameters) {
        if (null == this.inputParametersMeta || this.inputParametersMeta.isEmpty()) {
            return true;
        }
        for (Map.Entry<String, Property> entry : this.inputParametersMeta.entrySet()) {
            if (1 == entry.getValue().getIsRequired()) {
                verifyParameter(parameters, entry.getKey());
            }
        }
        return true;
    }

    // protected Boolean verifyParameters(Map<String, Property> parameters);

    protected void verifyParameter(Map<String, Property> parameters, String key) throws BusinessException {
        if (parameters != null &&
                (!parameters.containsKey(key)
                        || null == parameters.get(key)
                        || null == parameters.get(key).getValue())) {
            throw new BusinessException(SystemStatusCodeEnum.FAIL,
                    String.format("缺失参数数据[%s]，请检查算法里面此参数是否为必需。", key));
        }
    }

    /**
     * 返回默认的输出目录
     *
     * @return
     */
    protected String getDefaultOutputDirectory() {
        String defaultOutputDirectory = PathUtil.combine(Constant.GlobalSystemConfig.currentBaseWorkingDirectory(), Constant.GlobalSystemConfig.DIR_TEMP);
        DirectoryUtil.createDirectory(defaultOutputDirectory);
        return defaultOutputDirectory;
    }

    /**
     * 添加输入参数
     *
     * @param name
     * @param property
     */
    protected void addInputParameter(String name, Property property) {
        this.inputParametersMeta.put(name, property);
    }

    /**
     * 添加输出参数
     *
     * @param name
     * @param property
     */
    protected void addOutputParameter(String name, Property property) {
        this.outputParametersMeta.put(name, property);
    }

    /**
     * JSON文本转换为算法参数
     *
     * @param propertiesJson
     * @return
     */
    protected Map<String, Property> convertToParameters(String propertiesJson) {
        Map<String, JSONObject> map = JSON.parseObject(propertiesJson,
                new TypeReference<>() {
                });
        if (map.isEmpty()) {
            return null;
        }
        Map<String, Property> propertyMap = new HashMap<>();
        for (Map.Entry<String, JSONObject> entry : map.entrySet()) {
            propertyMap.put(entry.getKey(), entry.getValue().toJavaObject(Property.class));
        }
        return propertyMap;
    }
}
